'use strict';

var css = require('css');

var _require = require('glamor'),
    styleSheet = _require.styleSheet;

var getClassNames = function getClassNames(node) {
  var classNames = [];
  if (node.children && node.children.reduce) {
    classNames = node.children.reduce(function (acc, child) {
      return acc.concat(getClassNames(child));
    }, []);
  }
  var className = node.props && (node.props.className || node.props.class);
  if (className) {
    return classNames.concat(className.toString().split(' '));
  }
  return classNames;
};

var getStyles = function getStyles(classNames) {
  var styles = styleSheet.tags.map(function (tag) {
    return (/* istanbul ignore next */tag.textContent || ''
    );
  }).join('\n');
  var ast = css.parse(styles);
  ast.stylesheet.rules = ast.stylesheet.rules.filter(function (rule) {
    return rule.type === 'rule' && classNames.includes(rule.selectors[0].substring(1));
  });

  var ret = css.stringify(ast);
  return ret;
};

var serializer = {
  test(val) {
    return !val.withStyles && val.$$typeof === Symbol.for('react.test.json');
  },

  print(val, print) {
    var classNames = getClassNames(val);
    var styles = getStyles(classNames);
    val.withStyles = true;
    var printedVal = print(val);
    if (styles) {
      return `${styles}\n\n${printedVal}`;
    } else {
      return printedVal;
    }
  }
};

module.exports = serializer;